/*
  Copyright (C) 1999  Erik Thiele

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.
  
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
  
  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#ifndef __TEMPMATRIX_HPP
#define __TEMPMATRIX_HPP

#include "yyybinbuff.hpp"
#include "yyysendables.hpp"

template <class X>
class tempmatrix {
public:
  typedef int32 index_t; // must also be able to hold xsize * ysize !

  tempmatrix ();
  tempmatrix (const tempmatrix &o);
  tempmatrix & operator= (const tempmatrix &o);
  ~tempmatrix ();
  tempmatrix (const index_t xs_, const index_t ys_);

  inline index_t xsize () const { return xs; };
  inline index_t ysize () const { return ys; };
  inline const X &get (const index_t x, const index_t y) const { return matrix[x+y*xs]; };
  inline       X &get (const index_t x, const index_t y)       { return matrix[x+y*xs]; };

  inline const X &operator() (const index_t x, const index_t y) const { return get(x,y); };
  inline       X &operator() (const index_t x, const index_t y)       { return get(x,y); };

  void resize (const index_t nx, const index_t ny);
  void fill (const X &v);

  inline       X *getrawdatapointer()       { return matrix; };
  inline const X *getrawdatapointer() const { return matrix; };
  inline const int getcount() const { return count; };

private:
  index_t xs, ys, count;
  X *matrix;
};

template <class X>
tempmatrix<X>::tempmatrix ()
  : xs(0), ys(0), count(0), matrix(new X[0])
{
};

template <class X>
tempmatrix<X> & tempmatrix<X>::operator= (const tempmatrix &o)
{
  resize(o.xs, o.ys);
  for (index_t i=0;i<count;++i) matrix[i]=o.matrix[i];
  return *this;
};

template <class X>
tempmatrix<X>::~tempmatrix ()
{
  delete [] matrix;
};

template <class X>
tempmatrix<X>::tempmatrix (const index_t xs_, const index_t ys_)
  : xs(xs_), ys(ys_), count(xs_*ys_), matrix(new X[count])
{
};

template <class X>
void tempmatrix<X>::resize (const index_t nx, const index_t ny)
{
  if ((nx==xs) && (ny==ys)) return;
  delete [] matrix;
  xs=nx;
  ys=ny;
  count=xs*ys;
  matrix=new X [count];
};

template <class X>
void tempmatrix<X>::fill (const X &v)
{
  for (index_t i=0;i<count;++i) matrix[i]=const_cast<X&>(v); // FIXME this const_cast is WRONG. it's just there because ther is no
  // int[4] -> const int[4] operator. sucking.
};


template <class X>
void appendtobinbuffer (const tempmatrix<X> &a, yyybinbuff::binbuff &b)
{
  appendtobinbuffer(a.xsize(),b);
  appendtobinbuffer(a.ysize(),b);
  const X *const p = a.getrawdatapointer();
  for (tempmatrix<X>::index_t i=0;i<a.getcount();++i) appendtobinbuffer(p[i], b);
};

template <class X>
const yyybinbuff::binbuff::index readfrombinbuffer (tempmatrix<X> &a, const yyybinbuff::binbuff &b, const yyybinbuff::binbuff::index pos)
{
  yyybinbuff::binbuff::index read, mypos=pos;
  
  tempmatrix<X>::index_t xs, ys;

  read=readfrombinbuffer(xs,b,mypos);
  if (read<0) return read;
  mypos+=read;

  read=readfrombinbuffer(ys,b,mypos);
  if (read<0) return read;
  mypos+=read;

  //  if (b.size()-mypos<xs*ys*sizeof(X)) return -1;
  // FIXME that's ultra performance quick hack but dangerous if cell
  // encoding size changes!
  // and sizeof(X) has nothing to do with encoding size!!!
  // FIXME here must really be a possible solution to decide if -1 must be returned faster!

  a.resize(xs,ys);

  X *const p = a.getrawdatapointer();
  for (tempmatrix<X>::index_t i=0;i<a.getcount();++i) {
    read=readfrombinbuffer(p[i],b,mypos);
    if (read<0) return read;
    mypos+=read;
  };

  return mypos-pos;
};

#endif
